// DEMO7_12.CPP 24-bit bitmap loading demo

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  // just say no to MFC

#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h> // include directdraw

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

// default screen size
#define SCREEN_WIDTH    640  // size of screen
#define SCREEN_HEIGHT   480
#define SCREEN_BPP      32   // bits per pixel

#define WORLD_WIDTH     1920
#define WORLD_HEIGHT    1440
#define CAR_HEIGHT		120
#define CAR_WIDTH		120

#define TOP_CAR_BOUND     160  //the bound of the car in relation to the screen
#define BOTTOM_CAR_BOUND   320
#define LEFT_CAR_BOUND   213
#define RIGHT_CAR_BOUND   426

#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

#define ACCELERATION_RATE 45 //number of frames to wait before increasing speed

#define UP 1
#define RIGHT 2
#define DOWN 3
#define LEFT 4

// TYPES //////////////////////////////////////////////////////

// basic unsigned types
typedef unsigned short USHORT;
typedef unsigned short WORD;
typedef unsigned char  UCHAR;
typedef unsigned char  BYTE;

// container structure for bitmaps .BMP file
typedef struct BITMAP_FILE_TAG
{
	BITMAPFILEHEADER bitmapfileheader;  // this contains the bitmapfile header
	BITMAPINFOHEADER bitmapinfoheader;  // this is all the info including the palette
	PALETTEENTRY     palette[256];      // we will store the palette here
	UCHAR            *buffer;           // this is a pointer to the data
	
} BITMAP_FILE, *BITMAP_FILE_PTR;

// PROTOTYPES  //////////////////////////////////////////////

int Flip_Bitmap(UCHAR *image, int bytes_per_line, int height);

int Load_Bitmap_File(BITMAP_FILE_PTR bitmap, char *filename);

int Unload_Bitmap_File(BITMAP_FILE_PTR bitmap);

// MACROS /////////////////////////////////////////////////

// tests if a key is up or down
#define		VK_W	0x57
#define		VK_A	0x41
#define		VK_S	0x53
#define		VK_D	0x44

#define KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEYUP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

// initializes a direct draw struct
#define DDRAW_INIT_STRUCT(ddstruct) { memset(&ddstruct,0,sizeof(ddstruct)); ddstruct.dwSize=sizeof(ddstruct); }

// this builds a 16 bit color value in 5.5.5 format (1-bit alpha mode)
#define _RGB16BIT555(r,g,b) ((b & 31) + ((g & 31) << 5) + ((r & 31) << 10))

// this builds a 16 bit color value in 5.6.5 format (green dominate mode)
#define _RGB16BIT565(r,g,b) ((b & 31) + ((g & 63) << 5) + ((r & 31) << 11))

// this builds a 32 bit color value in A.8.8.8 format (8-bit alpha mode)
#define _RGB32BIT(a,r,g,b) ((b) + ((g) << 8) + ((r) << 16) + ((a) << 24))

// GLOBALS ////////////////////////////////////////////////

HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE hinstance_app      = NULL; // globally track hinstance

// directdraw stuff

LPDIRECTDRAW7         lpdd        = NULL;    // dd object
LPDIRECTDRAWSURFACE7  lpddsprimary = NULL;   // dd primary surface
LPDIRECTDRAWSURFACE7  lpddsback    = NULL;   // dd back surface
LPDIRECTDRAWSURFACE7  worldsurface = NULL;   // dd WORLD surface
LPDIRECTDRAWSURFACE7  worldsurface_buffer = NULL;   // dd WORLD to write on
LPDIRECTDRAWSURFACE7  title_screen= NULL;   
LPDIRECTDRAWPALETTE   lpddpal      = NULL;   // a pointer to the created dd palette
LPDIRECTDRAWCLIPPER   lpddclipper  = NULL;   // dd clipper
PALETTEENTRY          palette[256];          // color palette
PALETTEENTRY          save_palette[256];     // used to save palettes
DDSURFACEDESC2        ddsd;                  // a direct draw surface description struct
DDBLTFX               ddbltfx;               // used to fill
DDSCAPS2              ddscaps;               // a direct draw surface capabilities struct
HRESULT               ddrval;                // result back from dd calls
DWORD                 start_clock_count = 0; // used for timing

BITMAP_FILE           bitmap;                // holds the bitmap
LPDIRECTDRAWSURFACE7			  testcar[16];  //my test car
LPDIRECTDRAWSURFACE7  health_bar = NULL; 
LPDIRECTDRAWSURFACE7  health_border = NULL; 
LPDIRECTDRAWSURFACE7  small_car = NULL; 
LPDIRECTDRAWSURFACE7  castle_key[5]; 


LPDIRECTDRAWSURFACE7  sprites[1];
RECT				  sprite_on_world[1];

bool pixel_plot[2764800];
int num_pixels=2764800;


char buffer[80];                             // general printing buffer

int FRAME_TIMER =0;  //this is used to do things every x number of frames

DDCOLORKEY COLORKEY;
RECT VIEW_CLIP; //rectangle on game world
RECT DRUNK_VIEW_CLIP;
RECT WHOLE_SCREEN;   //rectangle covering whole screen
RECT CAR_LOCATION;  //the location of the car on the screen
int car_center_x = 320;
int car_center_y = 300;
RECT CAR_DIM; //dimentions of the car
int car_direction = 0; // 0-15 values representing car facing direction
int SPEED_MULTIPLIER = 0; //determines the speed of the vehicle
bool gas_last_frame = false;
bool reverse=false;
int accel_frame_counter =0;
RECT HEALTH_BAR;
RECT HEALTH_BORDER;
int drinking_timer=0;
int score=0;
int drinks_had=0;
bool stop_car;
RECT SMALL_CAR;
RECT KEY_LOCATION;
bool game_over=false;
struct car_points
{
	int x[10];
	int y[10];
};

struct d_effects
{
	bool veering;
	int veering_lr;
	bool accellerate;
	bool movetrack; //set whether the camera should follow drinking effects
	bool grow_shrink;
	bool accellerate_on;
};

struct liquor_store
{
	RECT dimentions;
	bool active_destination;
};

liquor_store liquor_castles[5];

d_effects drinking_flags;
int drink_effect_rate=30; //the rate of change in drinking effects
int drink_effect_grow_shrink_timer=0;
int drink_effect_strength=3; // execute effect every x number of frames
int GROW_SHRINK_RATE=30; //the number of frames in the cycle

car_points collision_points[16];
// FUNCTIONS ////////////////////////////////////////////////
void get_dx_dy(int facing_dir, int& dx, int& dy); //forward declaration
void move_view(int dx,int dy,RECT CAR_LOCATION);  //forward declaration


int Draw_Text_GDI(char *text, int x,int y,COLORREF color, LPDIRECTDRAWSURFACE7 lpdds)
{
// this function draws the sent text on the sent surface
// using color index as the color in the palette

HDC xdc; // the working dc

// get the dc from surface
if (FAILED(lpdds->GetDC(&xdc)))
return(0);

// set the colors for the text up
SetTextColor(xdc,color);

// set background mode to transparent so black isn't copied
SetBkMode(xdc, TRANSPARENT);

// draw the text a
TextOut(xdc,x,y,text,strlen(text));

// release the dc
lpdds->ReleaseDC(xdc);

// return success
return(1);
} // end Draw_Text_GDI


bool destination_reached()
{
	int active_castle=0;
	for (int i=0;i<5;i++)
	{
		if (liquor_castles[i].active_destination)
		{
			active_castle=i;
			break;
		}
	}
	if (car_center_x>=liquor_castles[i].dimentions.left &&
		car_center_x<=liquor_castles[i].dimentions.right &&
		car_center_y>=liquor_castles[i].dimentions.top &&
		car_center_y<=liquor_castles[i].dimentions.bottom)
		//car is in destination
	{
		int new_destination=0;
		do 
		{
			new_destination=rand()%5;
		}
		while (new_destination==active_castle); //keep going until we get a new location
		liquor_castles[active_castle].active_destination=false;
		liquor_castles[new_destination].active_destination=true;
		return true;
	}
	else
	{
		return false;
	}
}
void excecute_manual_camera_movement()
{
	int dx=0;
	int dy=0;
	if (KEYDOWN(VK_S) )
	{
		dy+=3;
	}	
	if (KEYDOWN(VK_W) )
	{
		dy-=3;
	}
	if (KEYDOWN(VK_A) )
	{
		dx-=3;
	}
	if (KEYDOWN(VK_D) )
	{
		dx+=3;
	}
	move_view(dx,dy,CAR_LOCATION);
}
void drinking_effects_set_view()
{
	if (drinking_flags.grow_shrink)
	{
		if (drink_effect_grow_shrink_timer<GROW_SHRINK_RATE)
		{//grow and return
			DRUNK_VIEW_CLIP=VIEW_CLIP;
			DRUNK_VIEW_CLIP.left+=drink_effect_grow_shrink_timer;
			DRUNK_VIEW_CLIP.right-=drink_effect_grow_shrink_timer;
			DRUNK_VIEW_CLIP.bottom-=drink_effect_grow_shrink_timer;
			DRUNK_VIEW_CLIP.top+=drink_effect_grow_shrink_timer;
		}
		/*else
		{//shrink
			DRUNK_VIEW_CLIP=VIEW_CLIP;
			DRUNK_VIEW_CLIP.left+=drink_effect_grow_shrink_timer-30;
			DRUNK_VIEW_CLIP.right-=drink_effect_grow_shrink_timer-30;
			DRUNK_VIEW_CLIP.bottom-=drink_effect_grow_shrink_timer-30;
			DRUNK_VIEW_CLIP.top+=drink_effect_grow_shrink_timer-30;
		}*/
		drink_effect_grow_shrink_timer++;
		drink_effect_grow_shrink_timer%=GROW_SHRINK_RATE;
	}
}
void drinking_effects_veer_accel(int &dx, int &dy)
{
	if (drinking_timer==0)
	{
		//change drinking effects here
		if (drinking_flags.veering)
		{
			int random=rand()%2;
			if (random==1)
			{
				drinking_flags.veering_lr=LEFT;
			}
			else
			{
				drinking_flags.veering_lr=RIGHT;
			}
		}
		if (drinking_flags.accellerate)
		{
			int random=rand()%2;
			if (random==1)
			{
				drinking_flags.accellerate_on=true;
			}
			else
			{
				drinking_flags.accellerate_on=false;
			}
		}
	}
	if (drinking_flags.veering && drinking_timer%drink_effect_strength==0)
	{
		if (drinking_flags.veering_lr==RIGHT)
		{
			get_dx_dy((car_direction+4)%16,dx,dy);
		}
		else //veer left
		{
			get_dx_dy((car_direction+12)%16,dx,dy);
		}
	}
	drinking_timer++;
	drinking_timer%=drink_effect_rate; //frequency of change in drinking effect
}
bool intersects(RECT rec1, RECT rec2, 
				bool &horizontal_containment,
				bool &vertical_containment,
				bool &horizontal_overlap,
				bool &vertical_overlap); //forward declaration

int map_pixel(int x, int y)
{
	return (y*1920 + x);
}

void health_effect(int amount);//forward declaration

bool collision_exists(int direction,RECT CAR_LOCATION/*bool pixel_plot[],car_points collision_points[]*/)
{
	int point_x=CAR_LOCATION.left;
	int point_y=CAR_LOCATION.top;

	for (int i=0;i<4;i++)
	{
		int x=collision_points[direction].x[i];
		int y=collision_points[direction].y[i];
		if (pixel_plot[map_pixel(x+point_x,y+point_y)] == true)
		{
			if (FRAME_TIMER%9==0)
			{
				health_effect(-1);
			}
			return true; // we have found a collision
		}
	}
	for (i=4;i<10;i++)
	{
		int x=collision_points[direction].x[i];
		int y=collision_points[direction].y[i];
		if (pixel_plot[map_pixel(x+point_x,y+point_y)] == true)
		{
			if (FRAME_TIMER%3==0)
			{
				health_effect(-1);
			}
			stop_car=true;
			return true; // we have found a collision
		}
	}

	return false;
}

bool map_sprite_to_screen(RECT sprite_on_map,RECT &sprite_on_screen, RECT &sprite_dimentions, RECT VIEW_CLIP)
{	// this function resolves the problem of sprites disappearing if not totally on screen
	bool horizontal_containment=false;	//returns true if there is something to display
	bool vertical_containment=false;
	bool horizontal_overlap=false;
	bool vertical_overlap=false;
	sprite_on_screen=sprite_on_map;
	if (intersects(sprite_on_map,VIEW_CLIP,
		horizontal_containment, 
		vertical_containment,
		horizontal_overlap,
		vertical_overlap) )
	{
		if (horizontal_containment && vertical_containment)
		{
			//return true; // the sprite is completely on the screen and does not need to be mapped
		}
		if (true||horizontal_overlap || vertical_overlap)
		{
			if (true||horizontal_overlap)
			{
				if(sprite_on_map.left<VIEW_CLIP.left)
				{
					int shift;
					shift=VIEW_CLIP.left-sprite_on_map.left;
					sprite_on_screen.left=shift;
					sprite_dimentions.left=shift;
				}
				if(sprite_on_map.right>VIEW_CLIP.right)
				{
					int shift;
					shift=VIEW_CLIP.right-sprite_on_map.right;
					sprite_on_screen.right=shift;
					sprite_dimentions.right=shift;
				}
			}
			if (true||vertical_overlap)
			{
				if(sprite_on_map.bottom>VIEW_CLIP.bottom)
				{
					int shift;
					shift=VIEW_CLIP.bottom-sprite_on_map.bottom;
					sprite_on_screen.bottom=shift;
					sprite_dimentions.bottom=shift;
				}
				if(sprite_on_map.top<VIEW_CLIP.top)
				{
					int shift;
					shift=VIEW_CLIP.top-sprite_on_map.top;
					sprite_on_screen.top=shift;
					sprite_dimentions.top=shift;
				}
			}			
		}
		else
		{
			//return false; //the sprite is completely off the screen and does not need to be mapped
		}
		
	}
	
	sprite_on_screen.left-=(VIEW_CLIP.left);
	sprite_on_screen.right-=(VIEW_CLIP.left);
	sprite_on_screen.top-=(VIEW_CLIP.top);
	sprite_on_screen.bottom-=(VIEW_CLIP.top);
	return true;
}
void health_effect(int amount)
{
	if (HEALTH_BAR.right-HEALTH_BAR.left+amount<=0)
	{
		HEALTH_BAR.right=HEALTH_BAR.left;
	}
	else
	{
		HEALTH_BAR.right+=amount;
	}
}
bool intersects(RECT rec1, RECT rec2, 
				bool &horizontal_containment,
				bool &vertical_containment,
				bool &horizontal_overlap,
				bool &vertical_overlap)
{
	if ( rec1.left<=rec2.right // rec1 overlaps rec2 from the right
		&& rec1.right>=rec2.left) // rec1 overlaps rec2 from the left
		
	{
		horizontal_overlap=true;
	}	
	else if ( rec2.left<=rec1.right
		&& rec2.right>=rec1.left) 
		
	{
		horizontal_overlap=true;
	}
	
	if(rec1.top<=rec2.bottom //rec1 overlaps rec2 from the bottom
		&& rec1.bottom>=rec2.top) //rec1 overlaps rec2 from the top
	{
		vertical_overlap=true;
	}
	else if(rec2.top<=rec1.bottom
		&& rec2.bottom>=rec1.top)
	{
		vertical_overlap=true;
	}
	
	if (rec1.left>=rec2.left && rec1.right<=rec2.right)
	{
		horizontal_containment=true;
	}
	else if (rec2.left>=rec1.left && rec2.right<=rec1.right)
	{
		horizontal_containment=true;
	}
	
	if (rec1.bottom>=rec2.bottom && rec1.top<=rec2.top)
	{
		vertical_containment=true;
	}
	else if (rec2.bottom>=rec1.bottom && rec2.top<=rec1.top)
	{
		vertical_containment=true;
	}
	
	if( (vertical_containment||horizontal_containment) ||
		( vertical_containment||horizontal_overlap) )
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool intersects(RECT rec1, RECT rec2)
{
	bool x,y,a,b; //null
	return intersects(rec1,rec2, x,y,a,b);
}
void set_car_collision_points(car_points direction[])
{
	direction[0].x[0]=75;
	direction[0].x[1]=97;
	direction[0].x[2]=20;
	direction[0].x[3]=43;
	direction[0].y[0]=21;
	direction[0].y[1]=44;
	direction[0].y[2]=76;
	direction[0].y[3]=99;
	direction[1].x[0]=83;
	direction[1].x[1]=102;
	direction[1].x[2]=18;
	direction[1].x[3]=38;
	direction[1].y[0]=26;
	direction[1].y[1]=59;
	direction[1].y[2]=63;
	direction[1].y[3]=96;
	direction[2].x[0]=22;
	direction[2].x[1]=22;
	direction[2].x[2]=98;
	direction[2].x[3]=98;
	direction[2].y[0]=42;
	direction[2].y[1]=80;
	direction[2].y[2]=42;
	direction[2].y[3]=80;           
	for (int i=0;i<3;i++)
	{
		direction[i].x[4]=(direction[i].x[0]+direction[i].x[1])/2;
		direction[i].y[4]=(direction[i].y[0]+direction[i].y[1])/2;
		//using midpoints to fill extra collision points on cars
		
		direction[i].x[5]=(direction[i].x[0]+direction[i].x[4])/2;
		direction[i].y[5]=(direction[i].y[0]+direction[i].y[4])/2;

		
		direction[i].x[6]=(direction[i].x[4]+direction[i].x[1])/2;
		direction[i].y[6]=(direction[i].y[4]+direction[i].y[1])/2;

		
		direction[i].x[7]=(direction[i].x[2]+direction[i].x[3])/2;
		direction[i].y[7]=(direction[i].y[2]+direction[i].y[3])/2;
	
		
		direction[i].x[8]=(direction[i].x[2]+direction[i].x[7])/2;
		direction[i].y[8]=(direction[i].y[2]+direction[i].y[7])/2;

		
		direction[i].x[9]=(direction[i].x[3]+direction[i].x[7])/2;
		direction[i].y[9]=(direction[i].y[3]+direction[i].y[7])/2;
	}
	//  completed 0 1 2
	direction[10]=direction[2];      //  completed  10
	direction[3] = direction[1]; //now flip horizontally....
	for (i=0;i<10;i++)
	{
		direction[3].y[i]= 120 - direction[3].y[i];
	}										// completed 3
	direction[4] = direction[0];
	for (i=0;i<10;i++)
	{
		direction[4].y[i]= 120 - direction[4].y[i]; //flip horizontally
	}										//completed 4
	direction[5] = direction[1];
	for (i=0;i<10;i++)
	{
		int temp=direction[5].x[i];    //rotate 90 degrees
		direction[5].x[i]= 120-direction[5].y[i];
		direction[5].y[i]=temp;
	}											//completed 5
	for (i=0;i<10;i++)
	{
		direction[6].x[i]=direction[2].y[i];     //completed 6
		direction[6].y[i]=direction[2].x[i];
	}
	direction[10] = direction[2];            // completed 10 14
	direction[14] = direction[6];
	
	for (i=0;i<10;i++)
	{
		direction[7].x[i]= 120 - direction[5].x[i];
		direction[7].y[i]= direction[5].y[i];       // completed 7
	}
	
	for (i=0;i<10;i++)
	{
		direction[8].x[i]= 120 - direction[4].x[i];
		direction[8].y[i]= direction[4].y[i];       // completed 8
	}
	
	for (i=0;i<10;i++)
	{
		direction[9].x[i]= 120 - direction[3].x[i];
		direction[9].y[i]= direction[3].y[i];       // completed 9
	}
	
	for (i=0;i<10;i++)
	{
		direction[11].x[i]= direction[9].x[i];
		direction[11].y[i]= 120 - direction[9].y[i];       // completed 11
	}
	
	for (i=0;i<10;i++)
	{
		direction[12].x[i]= direction[8].x[i];
		direction[12].y[i]= 120 - direction[8].y[i];       // completed 12
	}
	
	for (i=0;i<10;i++)
	{
		direction[13].x[i]= direction[7].x[i];
		direction[13].y[i]= 120 - direction[7].y[i];       // completed 13
	}
	
	for (i=0;i<10;i++)
	{
		direction[15].x[i]= direction[5].x[i];
		direction[15].y[i]= 120 - direction[5].y[i];       // completed 15
	}
}

void update_car_location()
{
	CAR_LOCATION.left=car_center_x -(CAR_WIDTH/2);// - VIEW_CLIP.left;
	CAR_LOCATION.right=car_center_x +(CAR_WIDTH/2)+(CAR_WIDTH%2);//- VIEW_CLIP.left;
	CAR_LOCATION.top=car_center_y -(CAR_HEIGHT/2);// - VIEW_CLIP.top;
	CAR_LOCATION.bottom=car_center_y +(CAR_HEIGHT/2)+(CAR_HEIGHT%2);// - VIEW_CLIP.top;
}

void get_dx_dy(int facing_dir, int& dx, int& dy)
{
	switch(facing_dir)
	{
	case 0: dx=2;
		dy=-2;
		break;
	case 1: dx=2;
		dy=-1;
		break;
	case 2: dx=2;
		dy=0;
		break;
	case 3: dx=2;
		dy=1;
		break;
	case 4: dx=2;
		dy=2;
		break;
	case 5: dx=1;
		dy=2;
		break;
	case 6: dx=0;
		dy=2;
		break;
	case 7: dx=-1;
		dy=2;
		break;
	case 8: dx=-2;
		dy=2;
		break;
	case 9: dx=-2;
		dy=1;
		break;
	case 10: dx=-2;
		dy=0;
		break;
	case 11: dx=-2;
		dy=-1;
		break;
	case 12: dx=-2;
		dy=-2;
		break;
	case 13: dx=-1;
		dy=-2;
		break;
	case 14: dx=0;
		dy=-2;
		break;
	case 15: dx=1;
		dy=-2;
		break;
	default: ;
	}
}

int Load_Bitmap(LPDIRECTDRAWSURFACE7 image,char *fname) {
	HDC hdc, hdcimage;	// device context of the surface and image
	
	BITMAP bitmap;		// bitmap object
	
	HBITMAP hbitmap;	// handle to the bitmap
	
	// load the bitmap and get the object
	
	hbitmap = (HBITMAP)LoadImage(NULL, fname, IMAGE_BITMAP, 0, 0,LR_LOADFROMFILE);
	if(hbitmap == NULL)
		return(0);
	// get a device context that is compatible with the drawing surface
	
	hdcimage = CreateCompatibleDC(NULL);
	if(hdcimage == NULL)
		return(0);
	// select the handle to the bitmap as the object for the device context of the image
	
	if(SelectObject(hdcimage, hbitmap) == NULL)
		return(0);
	// get the size of the object
	
	if(GetObject(hbitmap, sizeof(bitmap), &bitmap) == NULL)
		return(0);
	// get the device context for the ddraw surface
	
	if(image->GetDC(&hdc)!=DD_OK)
		return(0);
	// blit the image from the image's DC to the surface's DC
	
	if(BitBlt(hdc, 0, 0, bitmap.bmWidth, bitmap.bmHeight, hdcimage, 0, 0, SRCCOPY) == NULL)
		return(0);
	// release the device context and destroy the image device context
	
	if(image->ReleaseDC(hdc)!=DD_OK)
		return(0);
	if(DeleteDC(hdcimage) == NULL)
		return(0);
	return(1);
}  // end Load_Image()

int Load_Bitmap_File(BITMAP_FILE_PTR bitmap, char *filename)
{
	// this function opens a bitmap file and loads the data into bitmap
	
	int file_handle,  // the file handle
		index;        // looping index
	
	UCHAR   *temp_buffer = NULL; // used to convert 24 bit images to 16 bit
	OFSTRUCT file_data;          // the file data information
	
	// open the file if it exists
	if ((file_handle = OpenFile(filename,&file_data,OF_READ))==-1)
		return(0);
	
	// now load the bitmap file header
	_lread(file_handle, &bitmap->bitmapfileheader,sizeof(BITMAPFILEHEADER));
	
	// test if this is a bitmap file
	if (bitmap->bitmapfileheader.bfType!=BITMAP_ID)
	{
		// close the file
		_lclose(file_handle);
		
		// return error
		return(0);
	} // end if
	
	// now we know this is a bitmap, so read in all the sections
	
	// first the bitmap infoheader
	
	// now load the bitmap file header
	_lread(file_handle, &bitmap->bitmapinfoheader,sizeof(BITMAPINFOHEADER));
	
	// now load the color palette if there is one
	if (bitmap->bitmapinfoheader.biBitCount == 8)
	{
		_lread(file_handle, &bitmap->palette,MAX_COLORS_PALETTE*sizeof(PALETTEENTRY));
		
		// now set all the flags in the palette correctly and fix the reversed 
		// BGR RGBQUAD data format
		for (index=0; index < MAX_COLORS_PALETTE; index++)
		{
			// reverse the red and green fields
			int temp_color                = bitmap->palette[index].peRed;
			bitmap->palette[index].peRed  = bitmap->palette[index].peBlue;
			bitmap->palette[index].peBlue = temp_color;
			
			// always set the flags word to this
			bitmap->palette[index].peFlags = PC_NOCOLLAPSE;
		} // end for index
		
	} // end if
	
	// finally the image data itself
	_lseek(file_handle,-(int)(bitmap->bitmapinfoheader.biSizeImage),SEEK_END);
	
	// now read in the image, if the image is 8 or 16 bit then simply read it
	// but if its 24 bit then read it into a temporary area and then convert
	// it to a 16 bit image
	
	if (bitmap->bitmapinfoheader.biBitCount==8 || bitmap->bitmapinfoheader.biBitCount==16 || 
		bitmap->bitmapinfoheader.biBitCount==24)
	{
		// delete the last image if there was one
		if (bitmap->buffer)
			free(bitmap->buffer);
		
		// allocate the memory for the image
		if (!(bitmap->buffer = (UCHAR *)malloc(bitmap->bitmapinfoheader.biSizeImage)))
		{
			// close the file
			_lclose(file_handle);
			
			// return error
			return(0);
		} // end if
		
		// now read it in
		_lread(file_handle,bitmap->buffer,bitmap->bitmapinfoheader.biSizeImage);
		
	} // end if
	else
	{
		// serious problem
		return(0);
		
	} // end else
	
#if 0
	// write the file info out 
	printf("\nfilename:%s \nsize=%d \nwidth=%d \nheight=%d \nbitsperpixel=%d \ncolors=%d \nimpcolors=%d",
		filename,
		bitmap->bitmapinfoheader.biSizeImage,
		bitmap->bitmapinfoheader.biWidth,
		bitmap->bitmapinfoheader.biHeight,
		bitmap->bitmapinfoheader.biBitCount,
		bitmap->bitmapinfoheader.biClrUsed,
		bitmap->bitmapinfoheader.biClrImportant);
#endif
	
	// close the file
	_lclose(file_handle);
	
	// flip the bitmap
	Flip_Bitmap(bitmap->buffer, 
		bitmap->bitmapinfoheader.biWidth*(bitmap->bitmapinfoheader.biBitCount/8), 
		bitmap->bitmapinfoheader.biHeight);
	
	// return success
	return(1);
	
} // end Load_Bitmap_File

void move_view(int dx,int dy,RECT CAR_LOCATION)
{
	if (dx<0 && (VIEW_CLIP.right-CAR_LOCATION.right)<60)
	{
		dx/=2;
	}
	else if(dx>0 && (CAR_LOCATION.left)-(VIEW_CLIP.left)<60)
	{
		dx/=2;
	}
	if (dy<0 && (VIEW_CLIP.bottom)-(CAR_LOCATION.bottom)<60)
	{
		dy/=2;
	}
	else if(dy>0 && (CAR_LOCATION.top)-(VIEW_CLIP.top)<60)
	{
		dy/=2;
	}
	if ( VIEW_CLIP.left+dx > 0 
		&& VIEW_CLIP.right+dx <WORLD_WIDTH-1)
	{
		VIEW_CLIP.left+=dx;
		VIEW_CLIP.right+=dx;
	}
	else if (dx!=0) //move as close as possible to the edge without going over
	{
		if (dx>0)
			dx--;
		else if(dx<0)
			dx++;
		move_view(dx,0,CAR_LOCATION);
	}
	
	if ( VIEW_CLIP.top+dy > 0
		&& VIEW_CLIP.bottom+dy <WORLD_HEIGHT-1)
	{
		VIEW_CLIP.top+=dy;
		VIEW_CLIP.bottom+=dy;
	}
	else if (dy!=0)
	{
		if (dy>0)
			dy--;
		else if(dy<0)
			dy++;
		move_view(0,dy,CAR_LOCATION);
	}
}

///////////////////////////////////////////////////////////

DWORD Get_Clock(void)
{
	// this function returns the current tick count
	
	// return time
	return(GetTickCount());
	
} // end Get_Clock

///////////////////////////////////////////////////////////////
DWORD Start_Clock(void)
{
	// this function starts the clock, that is, saves the current
	// count, use in conjunction with Wait_Clock()
	
	return(start_clock_count = Get_Clock());
	
} // end Start_Clock

///////////////////////////////////////////////////////////////


DWORD Wait_Clock(DWORD count)
{
	// this function is used to wait for a specific number of clicks
	// since the call to Start_Clock
	
	while((Get_Clock() - start_clock_count) < count);
	return(Get_Clock());
	
} // end Wait_Clock

int Unload_Bitmap_File(BITMAP_FILE_PTR bitmap)
{
	// this function releases all memory associated with "bitmap"
	if (bitmap->buffer)
	{
		// release memory
		free(bitmap->buffer);
		
		// reset pointer
		bitmap->buffer = NULL;
		
	} // end if
	
	// return success
	return(1);
	
} // end Unload_Bitmap_File

///////////////////////////////////////////////////////////

int Flip_Bitmap(UCHAR *image, int bytes_per_line, int height)
{
	// this function is used to flip bottom-up .BMP images
	
	UCHAR *buffer; // used to perform the image processing
	int index;     // looping index
	
	// allocate the temporary buffer
	if (!(buffer = (UCHAR *)malloc(bytes_per_line*height)))
		return(0);
	
	// copy image to work area
	memcpy(buffer,image,bytes_per_line*height);
	
	// flip vertically
	for (index=0; index < height; index++)
		memcpy(&image[((height-1) - index)*bytes_per_line],
		&buffer[index*bytes_per_line], bytes_per_line);
	
	// release the memory
	free(buffer);
	
	// return success
	return(1);
	
} // end Flip_Bitmap

///////////////////////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
							UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
	// this is the main message handler of the system
	PAINTSTRUCT		ps;		// used in WM_PAINT
	HDC				hdc;	// handle to a device context
	char buffer[80];        // used to print strings
	
	// what is the message 
	switch(msg)
	{	
	case WM_CREATE: 
        {
			// do initialization stuff here
			// return success
			return(0);
		} break;
		
	case WM_PAINT: 
		{
			// simply validate the window 
			hdc = BeginPaint(hwnd,&ps);	 
			
			// end painting
			EndPaint(hwnd,&ps);
			
			// return success
			return(0);
		} break;
		
	case WM_DESTROY: 
		{
			
			// kill the application, this sends a WM_QUIT message 
			PostQuitMessage(0);
			
			// return success
			return(0);
		} break;
		
	default:break;
		
    } // end switch
	
	// process any messages that we didn't take care of 
	return (DefWindowProc(hwnd, msg, wparam, lparam));
	
} // end WinProc

///////////////////////////////////////////////////////////

int Game_Main(void *parms = NULL, int num_parms = 0)
{
	// this is the main loop of the game, do all your processing
	// here
	
	// make sure this isn't executed again
	if (window_closed)
		return(0);
	
	// for now test if user is hitting ESC and send WM_CLOSE
	if (KEYDOWN(VK_ESCAPE))
	{
		PostMessage(main_window_handle,WM_CLOSE,0,0);
		window_closed = 1;
	} // end if
	stop_car=false;
	int dx=0;
	int dy=0;
	get_dx_dy(car_direction,dx,dy);

	int saved_car_direction=car_direction;
	int saved_car_center_x = car_center_x; //save values incase we need to revert to them
	int saved_car_center_y = car_center_y;
	
	if (KEYDOWN(VK_UP) )// speed up the car
	{
		//LEFT_VIEW_CLIP;
		//RIGHT_VIEW_CLIP;
		if (!gas_last_frame)
		{
			accel_frame_counter=0; //this case the first frame of pressing gas
		}
		if (SPEED_MULTIPLIER<2 && accel_frame_counter%ACCELERATION_RATE==0)
		{
			SPEED_MULTIPLIER++;
			accel_frame_counter=1;
		}
		else if (gas_last_frame)
		{
			accel_frame_counter++;
			accel_frame_counter%=ACCELERATION_RATE;
		}
		gas_last_frame=true; 
	}
	else 
	{
		if (gas_last_frame)
		{
			accel_frame_counter=0;
		}
		else
		{
			accel_frame_counter++;
			accel_frame_counter%=ACCELERATION_RATE;
		}
		if (SPEED_MULTIPLIER>0 && accel_frame_counter%ACCELERATION_RATE==0)// slow down the car
		{
			SPEED_MULTIPLIER--;
			accel_frame_counter=1;
		}
		gas_last_frame=false; //mark gas as not pressed
	}

	if (KEYDOWN(VK_DOWN))// reverse
	{
		if (!gas_last_frame)
		{
			SPEED_MULTIPLIER=-1;
		}
		else
		{
			SPEED_MULTIPLIER=0;
		}
		reverse=true;
	}
	else
	{
		if (SPEED_MULTIPLIER<0)
		{
			SPEED_MULTIPLIER=0;
		}
		reverse=false;
	}

	if(game_over)
	{
		SPEED_MULTIPLIER=0;
	}
	
	if (!game_over && !(reverse && gas_last_frame)) //fixes glitch if user presses gas, brake, and turns
	{
		if ((SPEED_MULTIPLIER>0 || reverse )&&KEYDOWN(VK_LEFT) && (FRAME_TIMER%(25/SPEED_MULTIPLIER)==0))
		{		//turn the car								// adjust turn rate for speed
			if (!reverse)
			{
				car_direction= ((car_direction+15)%16);
			}
			else
			{
				car_direction= ((car_direction+1)%16);
			}	
		}
		if ((SPEED_MULTIPLIER>0 || reverse )&& KEYDOWN(VK_RIGHT)&& (FRAME_TIMER%(25/SPEED_MULTIPLIER)==0))
		{      //turn the car
			if (reverse)
			{
				car_direction= ((car_direction+15)%16);
			}
			else
			{
				car_direction= ((car_direction+1)%16);
			}	
		}
	}
	get_dx_dy(car_direction,dx,dy);
	
	int drink_dx=0;
	int drink_dy=0;
	drinking_effects_veer_accel(drink_dx,drink_dy); //set veer, and also set accellerate
	
	if( SPEED_MULTIPLIER!=0 )
	{
		int speedup_x=0;
		int speedup_y=0;
		if(drinking_flags.accellerate_on)
		{
			if (dx>0)
			{
				speedup_x=1;
			}
			else if (dx<0)
			{
				speedup_x=-1;
			}
			if (dy>0)
			{
				speedup_y=1;
			}
			else if (dy<0)
			{
				speedup_y=-1;
			}
		}
		else
		{
			speedup_x=0;
			speedup_y=0;
		}
		car_center_x+=(dx*SPEED_MULTIPLIER)+drink_dx+speedup_x;
		car_center_y+=(dy*SPEED_MULTIPLIER)+drink_dy+speedup_y;
	}
	else if (SPEED_MULTIPLIER<2 && !game_over)
	{ //deduct points for not moving fast enough
		score--;
	}
	
	update_car_location();
	
	if (collision_exists(car_direction,CAR_LOCATION))
	{
		car_center_x=saved_car_center_x;
		car_center_y=saved_car_center_y;
		car_direction=saved_car_direction;
		update_car_location();
		if (SPEED_MULTIPLIER!=0 && KEYDOWN(VK_LEFT) && (FRAME_TIMER%(25/SPEED_MULTIPLIER)==0) )
		{
			car_direction+=15;
			car_direction%=16;
		}
		if (SPEED_MULTIPLIER!=0 && KEYDOWN(VK_RIGHT) && (FRAME_TIMER%(25/SPEED_MULTIPLIER)==0))
		{
			car_direction++;
			car_direction%=16;
		}
	}
	else
	{
		if (drinking_flags.movetrack && SPEED_MULTIPLIER>0) //if camera is set to follow drunk effects
		{
			move_view(dx*SPEED_MULTIPLIER*2+drink_dx,dy*SPEED_MULTIPLIER*2+drink_dy,CAR_LOCATION);
		}
		else
		{
			move_view(dx*SPEED_MULTIPLIER*2,dy*SPEED_MULTIPLIER*2,CAR_LOCATION);
		}
	}

	excecute_manual_camera_movement();
	drinking_effects_set_view();
	if (destination_reached())
	{
		score+=200;
		drinks_had++;
	}

	if (drinks_had>=16)
	{
		drinking_flags.accellerate=true;
		drinking_flags.grow_shrink=true;
		drinking_flags.movetrack=false;
		drinking_flags.veering=true;
		int drink_effect_strength=2;
	}
	else if (drinks_had>=12)
	{
		drinking_flags.accellerate=true;
		drinking_flags.grow_shrink=true;
		drinking_flags.movetrack=false;
		drinking_flags.veering=true;
		int drink_effect_strength=3;
	}
	else if (drinks_had>=9)
	{
		drinking_flags.accellerate=true;
		drinking_flags.grow_shrink=true;
		drinking_flags.movetrack=false;
		drinking_flags.veering=false;
		int drink_effect_strength=15;
	}
	else if (drinks_had>=6)
	{
		drinking_flags.accellerate=true;
		drinking_flags.grow_shrink=true;
		drinking_flags.movetrack=true;
		drinking_flags.veering=false;
		int drink_effect_strength=45;
	}
	else if (drinks_had>=3)
	{
		drinking_flags.accellerate=true;
		drinking_flags.grow_shrink=false;
		drinking_flags.movetrack=true;
		drinking_flags.veering=false;
		int drink_effect_strength=400;
	}
	sprintf(buffer,"Score : %d    Drinks Had: %d " ,score,drinks_had);
	
	
	// copy the bitmap image to the primary buffer line by line
	// notice the 24 to 32 bit conversion pixel by pixel
	
	// lock the primary surface
	//lpddsback->Lock(NULL,&ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL);
	
	// get video pointer to primary surfce
	DWORD *primary_buffer = (DWORD *)ddsd.lpSurface;  
	
	worldsurface_buffer->Blt(NULL,worldsurface,NULL,DDBLT_WAIT,NULL); // copy the game world into back back buffer

	/*RECT dimentions=sprite_dimentions[0];
	RECT sprite_on_screen;
	map_sprite_to_screen(sprite_on_world[0],sprite_on_screen,dimentions,VIEW_CLIP);
	sprite_on_screen.top+=50;
	dimentions.top+=50;*/
	//worldsurface_buffer->Blt(&sprite_on_world[0],sprites[0],NULL,DDBLT_WAIT/*|DDBLT_KEYSRC*/,NULL);
	//lpddsback->Blt(NULL,sprites[0],NULL,DDBLT_WAIT/*|DDBLT_KEYSRC*/,NULL);

	worldsurface_buffer->Blt(&CAR_LOCATION,testcar[car_direction],NULL,DDBLT_WAIT|DDBLT_KEYSRC,NULL);
	if(drinking_flags.grow_shrink)
	{
		lpddsback->Blt(NULL,worldsurface_buffer,&DRUNK_VIEW_CLIP,DDBLT_WAIT,NULL);
	}
	else
	{
		lpddsback->Blt(NULL,worldsurface_buffer,&VIEW_CLIP,DDBLT_WAIT,NULL);
	}
	lpddsback->Blt(&HEALTH_BORDER,health_border,NULL,DDBLT_WAIT|DDBLT_KEYSRC,NULL);
	lpddsback->Blt(&HEALTH_BAR,health_bar,NULL,DDBLT_WAIT,NULL/*&health_fill_color*/);

	Draw_Text_GDI(buffer,50,13,RGB(255,255,255),lpddsback);
	sprintf(buffer,"Car Damage");
	Draw_Text_GDI(buffer,430,13,RGB(0,0,0),lpddsback);
	for (int i=0;i<5;i++)
	{
		if (liquor_castles[i].active_destination)
			break;
	}
	lpddsback->Blt(&KEY_LOCATION,castle_key[i],NULL,DDBLT_WAIT|DDBLT_KEYSRC,NULL);

	SMALL_CAR.left=(car_center_x/24)-1+280;
	SMALL_CAR.right=(car_center_x/24)+1+280;
	SMALL_CAR.top=(car_center_y/24)-1;
	SMALL_CAR.bottom=(car_center_y/24)+1;
	
	
	lpddsback->Blt(&SMALL_CAR,small_car,NULL,DDBLT_WAIT,NULL/*&health_fill_color*/);

	if (HEALTH_BAR.right-HEALTH_BAR.left <=0)
	{
		game_over=true;
	}
	if (game_over)
	{
		sprintf(buffer,"GAME OVER!  Your score is %d. Press ESC to exit... ",score);
		Draw_Text_GDI(buffer,150,240,RGB(255,0,0),lpddsback);
	}

	
	//DWORD *primary_buffer = (DWORD *)lpddsback->Lock(NULL,&ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL);     
	//*******************************************************************************
	//GAME WORLD BLITTING
	/*// process each line and copy it into the primary buffer  
	for ( int index_y = TOP_VIEW_CLIP; index_y < BOTTOM_VIEW_CLIP; index_y++)
    {
    for (int index_x = LEFT_VIEW_CLIP; index_x < RIGHT_VIEW_CLIP; index_x++)
	{
	// get BGR values
	UCHAR blue  = (bitmap.buffer[index_y*WORLD_WIDTH*3 + index_x*3 + 0]),
	green = (bitmap.buffer[index_y*WORLD_WIDTH*3 + index_x*3 + 1]),
	red   = (bitmap.buffer[index_y*WORLD_WIDTH*3 + index_x*3 + 2]);
	
	  // this builds a 32 bit color value in A.8.8.8 format (8-bit alpha mode)
	  DWORD pixel = _RGB32BIT(0,red,green,blue);
	  
        // write the pixel
        primary_buffer[(index_x-LEFT_VIEW_CLIP) + ((index_y-TOP_VIEW_CLIP)*ddsd.lPitch >> 2)] = pixel;
		
		  } // end for index_x
		  
			} // end for index_y
			//*******************************************************************************
			//CAR BLITTING
			// process each line and copy it into the primary buffer
			for ( index_y = 0; index_y < CAR_HEIGHT; index_y++)
			{
			for ( int index_x = 0; index_x < CAR_WIDTH; index_x++)
			{
			// get BGR values
			UCHAR blue  = (testcar.buffer[index_y*CAR_WIDTH*3 + index_x*3 + 0]),
			green = (testcar.buffer[index_y*CAR_WIDTH*3 + index_x*3 + 1]),
			red   = (testcar.buffer[index_y*CAR_WIDTH*3 + index_x*3 + 2]);
			
			  // this builds a 32 bit color value in A.8.8.8 format (8-bit alpha mode)
			  DWORD pixel = _RGB32BIT(0,red,green,blue);
			  
				// write the pixel
				// primary_buffer[(car_center_x-(CAR_WIDTH/2)-LEFT_VIEW_CLIP+index_x) + ((car_center_y-(CAR_WIDTH/2)-TOP_VIEW_CLIP+index_y)*ddsd.lPitch) >> 2] = pixel;
				//	primary_buffer[(index_x+100) + ((index_y+100)*ddsd.lPitch >> 2)] = _RGB32BIT(0,(UCHAR)NULL,(UCHAR)NULL,(UCHAR)NULL);
				primary_buffer[(index_x+100) + ((index_y+100)*ddsd.lPitch >> 2)] = pixel;
				} // end for index_x
				
				  } // end for index_y
	*/
	// now unlock the primary surface
	//lpddsprimary->Unlock(NULL);
	if (FAILED(lpddsprimary->Flip( NULL, DDFLIP_WAIT )))
		return(0);
	
	// do nothing -- look at pretty picture
	
	// return success or failure or your own return code here
	
	FRAME_TIMER++;
	FRAME_TIMER = FRAME_TIMER%(30);
	Wait_Clock(30);
	return(1);
	
} // end Game_Main

////////////////////////////////////////////////////////////

int Game_Init(void *parms = NULL, int num_parms = 0)
{
	// this is called once after the initial window is created and
	// before the main event loop is entered, do all your initialization
	// here
	
	// create IDirectDraw interface 7.0 object and test for error
	if (FAILED(DirectDrawCreateEx(NULL, (void **)&lpdd, IID_IDirectDraw7, NULL)))
		return(0);
	
	// set cooperation to full screen
	if (FAILED(lpdd->SetCooperativeLevel(main_window_handle,// DDSCL_NORMAL
		DDSCL_FULLSCREEN | DDSCL_ALLOWMODEX | 
		DDSCL_EXCLUSIVE | DDSCL_ALLOWREBOOT)))
		return(0);
	
	// set display mode to 640x480x8
	if (FAILED(lpdd->SetDisplayMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,0,0)))
		return(0);
	
	set_car_collision_points(collision_points);

	for (int i=0;i<5;i++)
	{
		liquor_castles[i].active_destination=false;
		if (i==3)
		{
			liquor_castles[i].active_destination=true;
		}
	}
	liquor_castles[0].dimentions.left=120;
	liquor_castles[0].dimentions.top=140;
	liquor_castles[0].dimentions.right=183;
	liquor_castles[0].dimentions.bottom=238;

	liquor_castles[1].dimentions.left=1142;
	liquor_castles[1].dimentions.top=440;
	liquor_castles[1].dimentions.right=1242;
	liquor_castles[1].dimentions.bottom=478;

	liquor_castles[2].dimentions.left=1432;
	liquor_castles[2].dimentions.top=631;
	liquor_castles[2].dimentions.right=1534;
	liquor_castles[2].dimentions.bottom=674;

	liquor_castles[3].dimentions.left=423;
	liquor_castles[3].dimentions.top=1026;
	liquor_castles[3].dimentions.right=532;
	liquor_castles[3].dimentions.bottom=1067;

	liquor_castles[4].dimentions.left=1421;
	liquor_castles[4].dimentions.top=1233;
	liquor_castles[4].dimentions.right=1459;
	liquor_castles[4].dimentions.bottom=1335;

	drinking_flags.veering=false;
	drinking_flags.veering_lr=LEFT;
	drinking_flags.movetrack=true;;
	drinking_flags.grow_shrink=false;
	drinking_flags.accellerate_on=false;
	drinking_flags.accellerate=false;
	drink_effect_rate=30; //the rate of change in drinking effects
	drink_effect_grow_shrink_timer=0;
	drink_effect_strength=2; // execute effect every x number of frames
	GROW_SHRINK_RATE=30; //the number of frames in the cycle
	
	car_center_x = 577;
	car_center_y = 847;
	car_direction = 10; // 0-15 values representing car facing direction
	SPEED_MULTIPLIER = 0; //determines the speed of the vehicle
	gas_last_frame = false;
	reverse=false;
	accel_frame_counter =0;
	drinking_timer=0;
	score=0;
	drinks_had=0;
	VIEW_CLIP.top=847-240;
	VIEW_CLIP.bottom=847+240;  // initialize viewing window to top left of world
	VIEW_CLIP.left=577-320;
	VIEW_CLIP.right=577+320;
	
	
	WHOLE_SCREEN.top=0;
	WHOLE_SCREEN.bottom=480;  //this is the size of the primary surface
	WHOLE_SCREEN.left=0;
	WHOLE_SCREEN.right=640;
	
	CAR_DIM.bottom=120;
	CAR_DIM.top=0;
	CAR_DIM.left=0;
	CAR_DIM.right=120;
	
	HEALTH_BAR.bottom=30;
	HEALTH_BAR.top=12;
	HEALTH_BAR.left=380;
	HEALTH_BAR.right=579;
	
	HEALTH_BORDER.bottom=31;
	HEALTH_BORDER.top=11;
	HEALTH_BORDER.left=379;
	HEALTH_BORDER.right=580;
	
	sprite_on_world[0].top=300;
	sprite_on_world[0].bottom=400;
	sprite_on_world[0].left=500;
	sprite_on_world[0].right=600;

	KEY_LOCATION.left=280;
	KEY_LOCATION.right=360;
	KEY_LOCATION.top=0;
	KEY_LOCATION.bottom=60;

	update_car_location();
	
	// clear ddsd and set size
	DDRAW_INIT_STRUCT(ddsd); 
	
	// set the backbuffer count field for double buffering
	ddsd.dwBackBufferCount = 1;
	
	// enable valid fields
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	
	// request primary surface
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	
	// create the primary surface
	if (FAILED(lpdd->CreateSurface(&ddsd, &lpddsprimary, NULL)))
		return(0);
	
	ddsd.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;
	
	lpddsprimary->GetAttachedSurface(&ddsd.ddsCaps, &lpddsback);

	
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH; 
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN; 
	ddsd.dwBackBufferCount = 0;
	ddsd.dwHeight = WORLD_HEIGHT; 
	ddsd.dwWidth = WORLD_WIDTH; 
	ddrval = lpdd->CreateSurface(&ddsd, &worldsurface, NULL); 
	Load_Bitmap(worldsurface,"bitmap24.bmp");

	
	ddrval = lpdd->CreateSurface(&ddsd, &worldsurface_buffer, NULL); //used as a back buffer to the back buffer
	
	Load_Bitmap(worldsurface_buffer,"collision_bitmap.bmp");
	worldsurface_buffer->Lock(NULL,&ddsd,NULL,NULL);
	DWORD *primary_buffer = (DWORD *)ddsd.lpSurface;
	for (i=0;i<2764800;i++)
	{
		if (primary_buffer[i]==RGB(1,1,1) )
		{
			pixel_plot[i]=true;
		}
	}
	worldsurface_buffer->Unlock(NULL); //use the worldsurface buffer to create pixel plot
	
	ddsd.dwHeight = 480; 
	ddsd.dwWidth = 640; 
	ddrval = lpdd->CreateSurface(&ddsd, &title_screen, NULL); 
	Load_Bitmap(title_screen,"title_screen.bmp");

	ddsd.dwHeight = 21; 
	ddsd.dwWidth = 202; 
	ddrval = lpdd->CreateSurface(&ddsd, &health_border, NULL); 
	Load_Bitmap(health_border,"health_border.bmp");
	
	ddsd.dwHeight = 10; 
	ddsd.dwWidth = 10; 
	ddrval = lpdd->CreateSurface(&ddsd, &health_bar, NULL); 
	Load_Bitmap(health_bar,"health_bar.bmp");

	ddsd.dwHeight = 1; 
	ddsd.dwWidth = 1; 
	ddrval = lpdd->CreateSurface(&ddsd, &small_car, NULL); 
	Load_Bitmap(small_car,"smallcar.bmp");

	ddsd.dwHeight = 60; 
	ddsd.dwWidth = 80; 
	ddrval = lpdd->CreateSurface(&ddsd, &castle_key[0], NULL); 
	Load_Bitmap(castle_key[0],"castle1.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &castle_key[1], NULL); 
	Load_Bitmap(castle_key[1],"castle2.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &castle_key[2], NULL); 
	Load_Bitmap(castle_key[2],"castle3.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &castle_key[3], NULL); 
	Load_Bitmap(castle_key[3],"castle4.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &castle_key[4], NULL); 
	Load_Bitmap(castle_key[4],"castle5.bmp");

	ddsd.dwHeight = 100; 
	ddsd.dwWidth = 100; 
	ddrval = lpdd->CreateSurface(&ddsd, &sprites[0], NULL); 
	Load_Bitmap(sprites[0],"testerclip.bmp");
	
	COLORKEY.dwColorSpaceLowValue = RGB(0,255,0);
	COLORKEY.dwColorSpaceHighValue = RGB(0,255,0);
	ddsd.dwHeight = CAR_HEIGHT; 
	ddsd.dwWidth = CAR_WIDTH; 
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[0], NULL); 
	Load_Bitmap(testcar[0],"testcar0.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[1], NULL); 
	Load_Bitmap(testcar[1],"testcar1.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[2], NULL); 
	Load_Bitmap(testcar[2],"testcar2.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[3], NULL); 
	Load_Bitmap(testcar[3],"testcar3.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[4], NULL); 
	Load_Bitmap(testcar[4],"testcar4.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[5], NULL); 
	Load_Bitmap(testcar[5],"testcar5.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[6], NULL); 
	Load_Bitmap(testcar[6],"testcar6.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[7], NULL); 
	Load_Bitmap(testcar[7],"testcar7.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[8], NULL); 
	Load_Bitmap(testcar[8],"testcar8.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[9], NULL); 
	Load_Bitmap(testcar[9],"testcar9.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[10], NULL); 
	Load_Bitmap(testcar[10],"testcar10.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[11], NULL); 
	Load_Bitmap(testcar[11],"testcar11.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[12], NULL); 
	Load_Bitmap(testcar[12],"testcar12.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[13], NULL); 
	Load_Bitmap(testcar[13],"testcar13.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[14], NULL); 
	Load_Bitmap(testcar[14],"testcar14.bmp");
	ddrval = lpdd->CreateSurface(&ddsd, &testcar[15], NULL); 
	Load_Bitmap(testcar[15],"testcar15.bmp");
	
	for (i=0;i<16;i++)
	{
		(testcar[i])->SetColorKey(DDCKEY_SRCBLT, &COLORKEY);
	}
	for (i=0;i<5;i++)
	{
		(castle_key[i])->SetColorKey(DDCKEY_SRCBLT, &COLORKEY);
	}
	
	health_border->SetColorKey(DDCKEY_SRCBLT, &COLORKEY);

	while(!KEYDOWN(VK_SPACE))
	{
		lpddsback->Blt(NULL,title_screen,NULL,DDBLT_WAIT,NULL);
		lpddsprimary->Flip(NULL,DDFLIP_WAIT);
		//Wait_Clock(30);
	}
	// return success or failure or your own return code here
	return(1);
	
} // end Game_Init

/////////////////////////////////////////////////////////////

int Game_Shutdown(void *parms = NULL, int num_parms = 0)
{
	// this is called after the game is exited and the main event
	// loop while is exited, do all you cleanup and shutdown here
	
	
	// first the palette
	if (lpddpal)
	{
		lpddpal->Release();
		lpddpal = NULL;
	} // end if
	
	// now the primary surface
	if (lpddsprimary)
	{
		lpddsprimary->Release();
		lpddsprimary = NULL;
	} // end if
	
	// now blow away the IDirectDraw4 interface
	if (lpdd)
	{
		lpdd->Release();
		lpdd = NULL;
	} // end if
	
	// unload the bitmap file, we no longer need it
	Unload_Bitmap_File(&bitmap);
	//Unload_Bitmap_File(&testcar);
	
	// return success or failure or your own return code here
	return(1);
	
} // end Game_Shutdown

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
				   HINSTANCE hprevinstance,
				   LPSTR lpcmdline,
				   int ncmdshow)
{
	
	WNDCLASSEX winclass; // this will hold the class we create
	HWND	   hwnd;	 // generic window handle
	MSG		   msg;		 // generic message
	HDC        hdc;      // graphics device context
	
	// first fill in the window class stucture
	winclass.cbSize         = sizeof(WNDCLASSEX);
	winclass.style			= CS_DBLCLKS | CS_OWNDC | 
		CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;
	winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);
	
	// save hinstance in global
	hinstance_app = hinstance;
	
	// register the window class
	if (!RegisterClassEx(&winclass))
		return(0);
	
	// create the window
	if (!(hwnd = CreateWindowEx(NULL,                  // extended style
		WINDOW_CLASS_NAME,     // class
		"DirectDraw 24-Bit Bitmap Loading", // title
		WS_POPUP | WS_VISIBLE,
		0,0,	  // initial x,y
		SCREEN_WIDTH,SCREEN_HEIGHT,  // initial width, height
		NULL,	  // handle to parent 
		NULL,	  // handle to menu
		hinstance,// instance of this application
		NULL)))	// extra creation parms
		return(0);
	
	// save main window handle
	main_window_handle = hwnd;
	
	// hide mouse
	ShowCursor(FALSE);
	// initialize game here
	Game_Init();
	
	// enter main event loop
	while(TRUE)
	{
		// test if there is a message in queue, if so get it
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			// test if this is a quit
			if (msg.message == WM_QUIT)
				break;
			
			// translate any accelerator keys
			TranslateMessage(&msg);
			
			// send the message to the window proc
			DispatchMessage(&msg);
		} // end if
		
		// main game processing goes here
		Game_Main();
		
	} // end while
	
	// closedown game here
	Game_Shutdown();
	
	// return to Windows like this
	return(msg.wParam);
	
} // end WinMain

///////////////////////////////////////////////////////////

